@@ -9,7 +9,7 @@ use 5.014;
 #use strict;
 #use warnings;
 
-our $VERSION = '0.22';
+our $VERSION = '0.23';
 
 our %TRUE_VALUES = (
                     'true' => 1,
@@ -22,7 +22,7 @@ sub new {
 
     my %data = (
         keep_unknown_categories => 0,
-        unknown_category_key    => 'other',
+        unknown_category_key    => 'Other',
 
         case_insensitive_cats => 0,
 
@@ -107,69 +107,70 @@ sub get_desktop_files {
     wantarray ? values(%table) : [values(%table)];
 }
 
-sub parse {
-    my ($self, $file_data, @desktop_files) = @_;
+# Used for unescaping strings
+my %Chr = (s => ' ', n => "\n", r => "\r", t => "\t", '\\' => '\\');
 
-    foreach my $desktop_file (@desktop_files) {
+sub parse_desktop_file {
+    my ($self, $desktop_file) = @_;
 
-        # Check the filename and skip it if it matches `skip_filename_re`
-        if (defined $self->{skip_filename_re}) {
-            substr($desktop_file, rindex($desktop_file, '/') + 1) =~ /$self->{skip_filename_re}/ && next;
-        }
+    # Check the filename and skip it if it matches `skip_filename_re`
+    if (defined $self->{skip_filename_re}) {
+        substr($desktop_file, rindex($desktop_file, '/') + 1) =~ /$self->{skip_filename_re}/ && return;
+    }
 
-        # Open and read the desktop file
-        sysopen my $desktop_fh, $desktop_file, 0 or next;
-        sysread $desktop_fh, (my $file), -s $desktop_file;
+    # Open and read the desktop file
+    sysopen my $desktop_fh, $desktop_file, 0 or return;
+    sysread $desktop_fh, (my $file), -s $desktop_file;
 
-        # Locate the "[Desktop Entry]" section
-        if ((my $index = index($file, "]\n", index($file, "[Desktop Entry]") + 15)) != -1) {
-            $file = substr($file, 0, $index);
-        }
+    # Locate the "[Desktop Entry]" section
+    if ((my $index = index($file, "]\n", index($file, "[Desktop Entry]") + 15)) != -1) {
+        $file = substr($file, 0, $index);
+    }
 
-        # Parse the entry data
-        my %info = $file =~ /$self->{_file_keys_re}/g;
+    # Parse the entry data
+    my %info = $file =~ /$self->{_file_keys_re}/g;
 
-        # Ignore the file when `NoDisplay` is true
-        if (exists $info{NoDisplay}) {
-            next if exists $TRUE_VALUES{$info{NoDisplay}};
-        }
+    # Ignore the file when `NoDisplay` is true
+    if (exists $info{NoDisplay}) {
+        return if exists $TRUE_VALUES{$info{NoDisplay}};
+    }
 
-        # Ignore the file when `Hidden` is true
-        if (exists $info{Hidden}) {
-            next if exists $TRUE_VALUES{$info{Hidden}};
-        }
+    # Ignore the file when `Hidden` is true
+    if (exists $info{Hidden}) {
+        return if exists $TRUE_VALUES{$info{Hidden}};
+    }
 
-        # If no 'Name' entry is defined, create one with the name of the file
-        $info{Name} //= substr($desktop_file, rindex($desktop_file, '/') + 1, -8);
+    # If no 'Name' entry is defined, create one with the name of the file
+    $info{Name} //= substr($desktop_file, rindex($desktop_file, '/') + 1, -8);
 
-        # Handle `skip_entry`
-        if (defined($self->{skip_entry}) and ref($self->{skip_entry}) eq 'ARRAY') {
-            my $skip;
-            foreach my $pair_ref (@{$self->{skip_entry}}) {
-                if (exists($info{$pair_ref->{key}}) and $info{$pair_ref->{key}} =~ /$pair_ref->{re}/) {
-                    $skip = 1;
-                    last;
-                }
+    # Unescape string escapes (\n, \t, etc.)
+    $info{$_} =~ s{\\(.)}{ $Chr{$1} // $1 }eg for (keys %info);
+
+    # Handle `skip_entry`
+    if (defined($self->{skip_entry}) and ref($self->{skip_entry}) eq 'ARRAY') {
+        foreach my $pair_ref (@{$self->{skip_entry}}) {
+            if (exists($info{$pair_ref->{key}}) and $info{$pair_ref->{key}} =~ /$pair_ref->{re}/) {
+                return;
             }
-            $skip and next;
         }
+    }
 
-        # Make user-defined substitutions
-        if (defined($self->{substitutions}) and ref($self->{substitutions}) eq 'ARRAY') {
-            foreach my $pair_ref (@{$self->{substitutions}}) {
-                if (exists $info{$pair_ref->{key}}) {
-                    if ($pair_ref->{global}) {
-                        $info{$pair_ref->{key}} =~ s/$pair_ref->{re}/$pair_ref->{value}/g;
-                    }
-                    else {
-                        $info{$pair_ref->{key}} =~ s/$pair_ref->{re}/$pair_ref->{value}/;
-                    }
+    # Make user-defined substitutions
+    if (defined($self->{substitutions}) and ref($self->{substitutions}) eq 'ARRAY') {
+        foreach my $pair_ref (@{$self->{substitutions}}) {
+            if (exists $info{$pair_ref->{key}}) {
+                if ($pair_ref->{global}) {
+                    $info{$pair_ref->{key}} =~ s/$pair_ref->{re}/$pair_ref->{value}/g;
+                }
+                else {
+                    $info{$pair_ref->{key}} =~ s/$pair_ref->{re}/$pair_ref->{value}/;
                 }
             }
         }
+    }
 
-        # Parse categories (and remove any duplicates)
-        my %categories;
+    # Parse categories (and remove any duplicates)
+    my %categories;
 
 #<<<
         @categories{
@@ -181,50 +182,65 @@ sub parse {
         } = ();
 #>>>
 
-        # Skip entry when there are no categories and `keep_unknown_categories` is false
-        scalar(%categories) or $self->{keep_unknown_categories} or next;
+    # Skip entry when there are no categories and `keep_unknown_categories` is false
+    %categories or $self->{keep_unknown_categories} or return;
 
-        # Remove `% ...` from the value of `Exec`
-        index($info{Exec}, ' %') != -1 and $info{Exec} =~ s/ +%.*//s;
+    # Store the categories
+    $info{Categories} = [keys %categories];
 
-        # Terminalize
-        if (    $self->{terminalize}
-            and defined($info{Terminal})
-            and exists($TRUE_VALUES{$info{Terminal}})) {
-            $info{Exec} = sprintf($self->{terminalization_format}, $self->{terminal}, $info{Exec});
+    # Remove `% ...` from the value of `Exec`
+    index($info{Exec}, ' %') != -1 and $info{Exec} =~ s/ +%.*//s;
+
+    # Terminalize
+    if (    $self->{terminalize}
+        and defined($info{Terminal})
+        and exists($TRUE_VALUES{$info{Terminal}})) {
+        $info{Exec} = sprintf($self->{terminalization_format}, $self->{terminal}, $info{Exec});
+    }
+
+    # Check and clean the icon name
+    if (exists $info{Icon}) {
+        my $icon = $info{Icon};
+
+        my $abs;
+        if (substr($icon, 0, 1) eq '/') {
+            if (-f $icon) {    # icon is specified as an absolute path
+                $abs = 1;
+            }
+            else {             # otherwise, take its basename
+                $icon = substr($icon, 1 + rindex($icon, '/'));
+            }
         }
 
-        # Check and clean the icon name
-        if (exists $info{Icon}) {
-            my $icon = $info{Icon};
-
-            my $abs;
-            if (substr($icon, 0, 1) eq '/') {
-                if (-f $icon) {    # icon is specified as an absolute path
-                    $abs = 1;
-                }
-                else {             # ... otherwise, take its basename
-                    $icon = substr($icon, 1 + rindex($icon, '/'));
-                }
-            }
-
-            # Remove any icon extension
-            if (!$abs) {
-                $icon =~ s/\.(?:png|jpe?g|svg|xpm)\z//i;
-            }
-
-            # Store the icon back into `%info`
-            $info{Icon} = $icon;
+        # Remove any icon extension
+        if (!$abs) {
+            $icon =~ s/\.(?:png|jpe?g|svg|xpm)\z//i;
         }
 
+        # Store the icon back into `%info`
+        $info{Icon} = $icon;
+    }
+
+    return %info;
+}
+
+sub parse {
+    my ($self, $hash_ref, @desktop_files) = @_;
+
+    foreach my $desktop_file (@desktop_files) {
+        my %info = $self->parse_desktop_file($desktop_file);
+
+        # Skip when %info is empty
+        %info || next;
+
         # Push the entry into its belonging categories
-        if (scalar(%categories)) {
-            foreach my $category (keys %categories) {
-                push @{$file_data->{$category}}, {map { $_ => $info{$_} } @{$self->{keys_to_keep}}};
+        if (exists($info{Categories}) and @{$info{Categories}}) {
+            foreach my $category (@{$info{Categories}}) {
+                push @{$hash_ref->{$category}}, {map { $_ => $info{$_} } @{$self->{keys_to_keep}}};
             }
         }
-        elsif ($self->{keep_unknown_categories}) {
-            push @{$file_data->{$self->{unknown_category_key}}}, {map { $_ => $info{$_} } @{$self->{keys_to_keep}}};
+        else {
+            push @{$hash_ref->{$self->{unknown_category_key}}}, {map { $_ => $info{$_} } @{$self->{keys_to_keep}}};
         }
     }
 }
@@ -304,7 +320,7 @@ is equivalent with:
         case_insensitive_cats   => 0,
 
         keep_unknown_categories => 0,
-        unknown_category_key    => 'other',
+        unknown_category_key    => 'Other',
       );
 
 =back
@@ -418,7 +434,25 @@ it returns an array reference containing the full names of the desktop files.
 
 =item $obj->parse(\%hash, @desktop_files)
 
-Parse a list of desktop files into a HASH ref.
+Parse a list of desktop files into a HASH ref, where the keys of the HASH are
+the categories from desktop files.
+
+=item $obj->parse_desktop_file($desktop_file)
+
+Parse a given desktop file and return a key-value list as a result.
+
+Example:
+
+    my %info = $obj->parse_desktop_file($desktop_file);
+
+where C<%info> might look something like this:
+
+    my %info = (
+        Name       => "...",
+        Exec       => "...",
+        Icon       => "...",
+        Categories => ["...", "...", "..."],
+    );
 
 =item $obj->parse_desktop_files()
 
